//******************************************************************************************************************
// functions.ino - part of talking_dtmf_relay.ino
// various data, voice and function commands.
//******************************************************************************************************************

void performCommand(byte DTMFdata)          // ---------- decode 4-bit code. Which button was pressed? Act. ----------
{
  switch (DTMFdata)
  {
    case 0:  //D
      break;
    case 1:  //1 ----------- Say Callsign
      delay(1500);
      digitalWrite(pttRelayPin, HIGH);              //key radio PTT 
      delay(500);                                   //wait 1/2 second
      sayCall();                                    //say callsign
      digitalWrite(pttRelayPin, LOW);               //release radio PTT
      break;
    case 2:  //2 ----------- Say Temperature
      temperature = getTemperature();               //call function to get the temperature
      sayTemperature(temperature, false);           //call function to say the temperature
      break;
    case 3:  //3 ----------- Say Supply Voltage
      volts = getVolts();                           //call function to get the voltage
      sayVolts(volts, false);                       //call function to say the voltage
      break;
    case 4:  //4 ----------
      break;
    case 5:  //5 ----------
      break;
    case 6:  //6 ----------
      break;
    case 7:  //7 ----------
      break;
    case 8:  //8 ----------
      break;
    case 9:  //9 ----------
     break;
    case 10:  //0 ----------- Say Danger Message
      sayDangerMsg();                               //call function to say funny message
      break;
    case 11:  //* ----------- Turn OFF Relay
      digitalWrite(controlledRelayPin, LOW);        //turn the controlled relay off
      delay(1500);                                  //wait 1.5 seconds
      digitalWrite(pttRelayPin, HIGH);              //key the radio PTT
      delay(500);                                   //wait 1/2 second
      voice.say(spTHE);                             //say the message
      voice.say(spMACHINE);
      voice.say(spIS);
      voice.say(spSHUT);
      voice.say(spOFF);
      digitalWrite(pttRelayPin, LOW);               //release the radio PTT
      break;
    case 12:  //# ----------- Turn ON Relay
      digitalWrite(controlledRelayPin, HIGH);       //turn the controlled relay on
      delay(1500);                                  //wait 1.5 seconds
      digitalWrite(pttRelayPin, HIGH);              //key the radio PTT
      delay(500);                                   //wait 1/2 second
      voice.say(spTHE);                             //say the message
      voice.say(spMACHINE);
      voice.say(spIS);
      voice.say(spON);
      digitalWrite(pttRelayPin, LOW);               //release the radio PTT
      break;
    case 13:  //A ----------
      break;
    case 14:  //B ----------
      break;
    case 15:  //C ----------
      break;
  }
}

void checkTemperature()                     // ---------- check temperature alarm limits, warn if exceeded ----------
{
  bool id = false;                                  //should we id
  temperature = getTemperature();                   //call function to check temperature
  if(temperature >= maxTemp || temperature <= minTemp)//warn if out of range
  {
    warningRepeats = warningRepeats + 1;            //increment counter
    delay(20);                                      //delay needed to execute correctly
    if(warningRepeats > maxWarningRepeats)          //only warn maxWarningRepeats times
    {
      if(currentMillis > (lastWarningTime + warningInterval)) //wait for warningInterval to expire
      {
        warningRepeats = 0;                         //reset warning count
      }
    }
    else
    {
      if(warningRepeats == 1)                       //id at begining of each recount
      {
        id = true;
      }
      sayTemperature(temperature, id);              //call function to say temperature
      lastWarningTime = currentMillis;              //remember the time
    }
  }   
}

float getTemperature()                      // ---------- read temperature from LM34 ----------
{
  int voltage = analogRead(tempPin);                //get voltage from sensor
  float temp = ((voltage/1024.0)*5000)/10;          //calculate temperature in degrees f
  return temp;
}

void sayTemperature(float temp, bool id)    // ---------- say temperature ----------
{
  Serial.print ("temperature = ");
  Serial.println(temp);
  delay(1500);
  digitalWrite(pttRelayPin, HIGH);
  delay(500);
  if(id == true){                                   //need to say id first
    if(temperature >= maxTemp){                     //if max exceeded, make it interesting
      sayDangerMsg();
    }
    else
    {
      sayCall();
      voice.say(spWARNING);                         //add warning too
    }
  }
  voice.say(spTHE);
  voice.say(spTEMPERATURE);
  voice.say(spIS);
  sayFloatNumber(temp);                             //parse floating point number and say it
  voice.say(spDEGREES);
  digitalWrite(pttRelayPin, LOW);   
}

void checkVolts()                           // ---------- check voltage alarm limits, warn if exceeded ----------
{
  bool id = false;                                  //should we id
  volts = getVolts();                               //call function to get voltage
  if(volts >= maxVolts || volts <= minVolts)        //warn if out of range
  {
    warningRepeats = warningRepeats + 1;
    delay(20);                                      //delay needed to execute correctly
    if(warningRepeats > maxWarningRepeats)
    {
      if(currentMillis > (lastWarningTime + warningInterval))
      {
        warningRepeats = 0; 
      }
    }
    else
    {
      if(warningRepeats == 1)
      {
        id = true;
      }
      sayVolts(volts, id);
      lastWarningTime = currentMillis;      
    }
  } 
}

float getVolts()                            // ---------- read voltage ----------
{
  int voltage = analogRead(voltPin);                //get voltage
  float volts = ((float)voltage/1023) * 16.6;       //scale it so 16.6(maximum) volts = 1023
  return volts;
}

void sayVolts(float vlts, bool id)          // ---------- say voltage ----------
{
  Serial.print("Volts = ");
  Serial.println(vlts);
  delay(1500);
  digitalWrite(pttRelayPin, HIGH);
  delay(500);
  if(id == true){
    sayCall();
    voice.say(spWARNING);
  }
  voice.say(spTHE);
  voice.say(spPOWER);
  voice.say(spLEVEL);
  voice.say(spIS);
  sayFloatNumber(vlts);        
  voice.say(spVOLTS);
  digitalWrite(pttRelayPin, LOW);   
}

void sayCall()                              // ---------- say callsign ----------
{
  voice.say(spW);                                   //NOTE: change to your call sign
  voice.say(spFIVE);                                //you may need to add letters & numbers to words.h
  voice.say(spJ);
  voice.say(spD);
  voice.say(spX);  
}

void saySignOn()                            // ---------- say sign on message ----------
{
  delay(1500);
  digitalWrite(pttRelayPin, HIGH);
  delay(500);
  sayCall();
  voice.say(spIS);
  voice.say(spON);
  voice.say(spTHE);
  delay(150);
  voice.say(spAIR);
  delay(500);
  digitalWrite(pttRelayPin, LOW);
}
void sayDangerMsg()                         // ---------- say funny message ----------
{
  Serial.println("danger msg");
  delay(1500);
  digitalWrite(pttRelayPin, HIGH);
  delay(500);
  voice.say(spDANGER);
  sayCall();
  delay(500);
  voice.say(spTHE);
  voice.say(spRADIO);
  voice.say(spIS);
  voice.say(spON);
  voice.say(spFIRE);
  digitalWrite(pttRelayPin, LOW);
}

void sayFloatNumber(float num)              // ---------- Split floating decimal number into two parts and call sayNumber to speak it ----------
{
  int whole = num;                                  //get whole number only
  sayNumber(whole);                                 //say whole number
  int deci = (num - whole) * 100;                   //get decimal portion of number
  if(deci > 0){
    voice.say(spPOINT);
    sayNumber(deci);
  }
}

void sayNumber(long n)                      // ---------- Say any number between -999,999 and 999,999 ----------
{
  if (n<0) {
    voice.say(spMINUS);
    sayNumber(-n);
  } else if (n==0) {
    voice.say(spZERO);
  } else {
    if (n>=1000) {
      int thousands = n / 1000;
      sayNumber(thousands);
      voice.say(spTHOUSAND);
      n %= 1000;
      if ((n > 0) && (n<100)) voice.say(spAND);
    }
    if (n>=100) {
      int hundreds = n / 100;
      sayNumber(hundreds);
      voice.say(spHUNDRED);
      n %= 100;
      if (n > 0) voice.say(spAND);
    }
    if (n>19) {
      int tens = n / 10;
      switch (tens) {
        case 2: voice.say(spTWENTY); break;
        case 3: voice.say(spTHIR_); voice.say(spT); break;
        case 4: voice.say(spFOUR); voice.say(spT);  break;
        case 5: voice.say(spFIF_);  voice.say(spT); break;
        case 6: voice.say(spSIX);  voice.say(spT); break;
        case 7: voice.say(spSEVEN);  voice.say(spT); break;
        case 8: voice.say(spEIGHT);  voice.say(spT); break;
        case 9: voice.say(spNINE);  voice.say(spT); break;
      }
      n %= 10;
    }
    switch(n) {
      case 1: voice.say(spONE); break;
      case 2: voice.say(spTWO); break;
      case 3: voice.say(spTHREE); break;
      case 4: voice.say(spFOUR); break;
      case 5: voice.say(spFIVE); break;
      case 6: voice.say(spSIX); break;
      case 7: voice.say(spSEVEN); break;
      case 8: voice.say(spEIGHT); break;
      case 9: voice.say(spNINE); break;
      case 10: voice.say(spTEN); break;
      case 11: voice.say(spELEVEN); break;
      case 12: voice.say(spTWELVE); break;
      case 13: voice.say(spTHIR_); voice.say(sp_TEEN); break;
      case 14: voice.say(spFOUR); voice.say(sp_TEEN);break;
      case 15: voice.say(spFIF_); voice.say(sp_TEEN); break;
      case 16: voice.say(spSIX); voice.say(sp_TEEN); break;
      case 17: voice.say(spSEVEN); voice.say(sp_TEEN); break;
      case 18: voice.say(spEIGHT); voice.say(sp_TEEN); break;
      case 19: voice.say(spNINE); voice.say(sp_TEEN); break;
    }
  }
}

